﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;

using System.Web.Configuration;
using System.Data.SqlClient;

namespace com.WizardWu
{
    public class SqlServer2005_DAL2
    {
        public int _totalRecords = 0;   // 全域變數。從 Stored Procedure 回傳的單一數值，取得符合查詢條件的資料列總筆數

        // 取得 Web.config 連線字串的名稱
        public string strConnString = System.Configuration.ConfigurationManager.ConnectionStrings["Northwind_SqlServer2005"].ToString();
        
        public SqlServer2005_DAL2()
        {}

        #region ****** GridView 在 DAL 層，執行 SELECT 多筆記錄時的函數 (搭配 Stored Procedure 做「分頁」處理) ******

        // 搭配 Stored Procedure 做「分頁」處理
        // SelectDataSetBindGridView() 的參數: (SQL陳述式、SQL參數值、WHERE條件過濾式(包含使用者輸入的關鍵字)、GridView 目前這一頁的起始列索引, GridView 每頁要顯示的筆數, 排序欄位名稱)
        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, true)]  // true 表示為 ObjectDataSource 要 SELECT 時的預設值
        public DataSet selectDataSetBindGridView_DAL2(string strTableName, string strColumnName, string strSqlWhere, string strDefaultOrderColumn, int startRowIndex, int maxinumRows, string sortExpression)
        {
            SqlConnection sqlConn = null;
            SqlCommand sqlCmd = null;
            SqlDataAdapter sqlAdapter = null;
            DataSet ds = null;
            string strOrderBy = null;       // 排序欄位 + 排序方式 (昇幕或降幕)

            try
            {
                sqlConn = new SqlConnection(strConnString);
                sqlConn.Open();

                if (sqlConn.State == ConnectionState.Open)
                {
                    ds = new DataSet();

                    // 排序欄位、排序方式 (昇幕或降幕)
                    if (string.IsNullOrEmpty(sortExpression))
                    {
                        // 排序欄位。第一次進入頁面時，sortExpression 為空值，以開發人員在 DAL 層指定的欄位 (本例中為 OrderID) 由大到小排序
                        strOrderBy = strDefaultOrderColumn + " DESC";    // GridView 預設排序方式，由大到小排序
                    }
                    else if ((!string.IsNullOrEmpty(sortExpression)) && (sortExpression.IndexOf(" DESC") == -1))
                    {
                        // 排序欄位。若使用者按下了 GridView 某個欄位的 title，此時 sortExpression 有值，且要由小到大排序
                        strOrderBy = sortExpression + " ASC";
                    }
                    else if ((!string.IsNullOrEmpty(sortExpression)) && (sortExpression.IndexOf(" DESC") != -1))
                    {
                        // 刪除 ObjectDataSource 自動在最後面加的「 DESC」字樣(5個字元)
                        sortExpression = sortExpression.Remove(sortExpression.Length - 5, 5);

                        // 排序欄位。若使用者按下了 GridView 某個欄位的 title，此時 sortExpression 有值，且要由大到小排序
                        strOrderBy = sortExpression + " DESC";
                    }

                    // SELECT SQL 的 WHERE 條件 (包含使用者輸入的模糊查詢關鍵字)
                    if (string.IsNullOrEmpty(strSqlWhere))
                    {
                        strSqlWhere = " 1=1 ";      // 若沒加這句，則當 WHERE 條件為空值時(使用者未輸入搜尋關鍵字)，丟到 Stored Procedure 裡後會 Error
                    }

                    sqlCmd = new SqlCommand("dbo.pager_SqlServer2005", sqlConn);                    // 系統中每支程式共用的 Stored Procedure 名稱
                    sqlCmd.CommandType = CommandType.StoredProcedure;                               // 資料來源為 Stored Procedure
                    sqlCmd.Parameters.Add(new SqlParameter("@StartRowIndex", startRowIndex));       // 目前頁面，要撈的資料，的第一筆資料的索引 (第一筆為 0，依序加 1，依此類推)
                    sqlCmd.Parameters.Add(new SqlParameter("@PageSize", maxinumRows));              // GridView 每頁要顯示的資料筆數
                    sqlCmd.Parameters.Add(new SqlParameter("@tableName", strTableName));
                    sqlCmd.Parameters.Add(new SqlParameter("@columnName", strColumnName));
                    sqlCmd.Parameters.Add(new SqlParameter("@sqlWhere", strSqlWhere));
                    sqlCmd.Parameters.Add(new SqlParameter("@orderBy", strOrderBy));

                    sqlCmd.Parameters.Add(new SqlParameter("@rowCount", SqlDbType.Int));            // Stored Procedure 會傳回一個數字(符合查詢條件的資料總筆數)，
                    sqlCmd.Parameters["@rowCount"].Direction = ParameterDirection.Output;           // 用來在 GridView 下方的「頁碼列」顯示頁碼之用(Ex: 1 2 3 4 ... 55)


                    sqlAdapter = new SqlDataAdapter(sqlCmd);
                    sqlAdapter.Fill(ds);

                    // 總筆數 (要回傳至前端 .aspx，給 GridView 下方的「頁碼列」顯示用)
                    _totalRecords = Convert.ToInt32(sqlCmd.Parameters["@rowCount"].Value);
                    // 存至 Session，以便在 .aspx.cs 中取用後，用來在 GridView 下方的「頁碼列」中顯示頁碼之用
                    HttpContext.Current.Session["s_RecordTotalCount"] = _totalRecords;
                    Console.Write("tc: " + _totalRecords);


                    // 實際從資料庫撈回來，儲存在 AP Server 記憶體中 DataSet/DataTable 的資料筆數
                    int jj = ds.Tables[0].Rows.Count;     // GridView 每頁要呈現的資料筆數
                    Console.Write(jj);
                }
            }
            catch (SqlException ex)
            {
                Console.WriteLine(ex.ToString());
                throw;
            }
            catch (Exception ex)
            {
                throw new Exception("發生 SqlClient 以外的資料庫錯誤: " + ex.Message);
            }
            finally
            {
                sqlCmd.Dispose();
                sqlAdapter.Dispose();
                ds.Dispose();
                if (sqlConn.State == ConnectionState.Open)
                {
                    sqlConn.Close();
                }
                sqlConn.Dispose();
            }

            return ds;

        } // end of selectDataSetBindGridView_DAL2()


        // 從 Stored Procedure 回傳的一個整數，為符合查詢條件的資料總筆數，用來在 GridView 下方的「頁碼列」顯示頁碼之用(Ex: 1 2 3 4 ... 55)
        public int sendRecordCountToDAL()
        {
            return _totalRecords;
        }

        #endregion region ****** GridView 在 DAL 層，執行 SELECT 多筆記錄時的函數 (搭配 Stored Procedure 做「分頁」處理) ******




        #region ****** 取得 Web.config 第一組連線字串的名稱 ******
        //public static string GetFirstConnStringName()
        //{
        //    // Get the connectionStrings key,value pairs collection.
        //    ConnectionStringSettingsCollection connectionStrings = WebConfigurationManager.ConnectionStrings as ConnectionStringSettingsCollection;

        //    // Get the collection enumerator.
        //    IEnumerator connectionStringsEnum = connectionStrings.GetEnumerator();

        //    // Loop through the collection and display the connectionStrings key, value pairs.
        //    int i = 0;
        //    string strFirstConnStringName = "";
        //    //Response.Write("[Display connectionStrings]");
        //    while (connectionStringsEnum.MoveNext())
        //    {
        //        i += 1;
        //        if (i == 1)
        //        {
        //            // 傳回第一組連線字串的 name
        //            strFirstConnStringName = connectionStrings[i].Name;
        //            break;
        //        }
        //    }
        //    return strFirstConnStringName;
        //}
        #endregion ****** 取得 Web.config 第一組連線字串的名稱 ******


    } // end of class WizardbAccess
} // end of namespace