﻿using System;
using System.Web.UI;
using System.Web.UI.WebControls;

/// <summary>
/// WizardPagerLinkButton 的摘要描述
/// </summary>
/// 

namespace com.WizardWu.Controls
{
    public class WizardPagerLinkButton : LinkButton
    {
        // 分頁-UI
        private readonly IPostBackContainer _container;
        private bool _enableCallback;
        private string _callbackArgument;

        // 分頁-UI
        // Constructor
        public WizardPagerLinkButton(IPostBackContainer container)
        {
            _container = container;
        }


        // 分頁-UI
        public void EnableCallback(string argument)
        {
            _enableCallback = true;
            _callbackArgument = argument;
        }


        // 分頁-UI
        public override bool CausesValidation
        {
            get { return false; }
            set { throw new ApplicationException("Cannot set validation on pager buttons"); }
        }


        // 分頁-UI
        protected override void Render(HtmlTextWriter writer)
        {
            SetCallbackProperties();
            base.Render(writer);
        }


        // 分頁-UI
        private void SetCallbackProperties()
        {
            if (_enableCallback)
            {
                ICallbackContainer container = _container as ICallbackContainer;
                if (container != null)
                {
                    string callbackScript = container.GetCallbackScript(this, _callbackArgument);
                    if (!string.IsNullOrEmpty(callbackScript)) OnClientClick = callbackScript;
                }
            }
        }


    } // end of class
} // end of namespace
