﻿using System;
using System.Globalization;
using System.Reflection;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Web;
using Microsoft.VisualBasic;
using System.Drawing;

/// <summary>
/// WizardGridView 的摘要描述
/// </summary>
/// 

namespace com.WizardWu.Controls
{
    [ToolboxData("<{0}:WizardGridView runat=server></{0}:WizardGridView>")]
    public class WizardGridView : System.Web.UI.WebControls.GridView
    {
        //public WizardGridView()
        //{ }

        //private WizardGridView.PagingStyleCollection _pagingStyle;
        private int? _recordCount = null;       // ObjectDataSource 裡撈得的總筆數
        //private int _recordCount = null;       // ObjectDataSource 裡撈得的總筆數


        // 光棒
        private System.Drawing.Color FOddRowBackColor = System.Drawing.Color.Empty;         // 單數列的背景顏色
        private System.Drawing.Color FEvenRowBackColor = System.Drawing.Color.Empty;        // 雙數列的背景顏色
        private System.Drawing.Color FMouseOverBackColor = System.Drawing.Color.Empty;      // 滑鼠移至資料列上時的背景顏色

        // 光棒 - 單數列的背景顏色
        [System.ComponentModel.DefaultValue(""),
        System.ComponentModel.Category("Appearance"),
        System.ComponentModel.Description("單數列的背景顏色")]
        public System.Drawing.Color Wizard_OddRowBackColor
        {
            get
            {
                return FOddRowBackColor;
            }
            set
            {
                FOddRowBackColor = value;
            }
        }


        // 光棒 - 雙數列的背景顏色
        [System.ComponentModel.DefaultValue(""),
        System.ComponentModel.Category("Appearance"),
        System.ComponentModel.Description("雙數列的背景顏色")]
        public System.Drawing.Color Wizard_EvenRowBackColor
        {
            get
            {
                return FEvenRowBackColor;
            }
            set//(value As System.Drawing.Color)
            {
                FEvenRowBackColor = value;
            }
        }


        // 光棒 - 滑鼠移至資料列上時的背景顏色
        [System.ComponentModel.DefaultValue(""),
        System.ComponentModel.Category("Appearance"),
        System.ComponentModel.Description("滑鼠移至資料列上時的背景顏色")]
        public System.Drawing.Color Wizard_MouseOverBackColor
        {
            get
            {
                return FMouseOverBackColor;
            }
            set//(value As System.Drawing.Color)
            {
                FMouseOverBackColor = value;
            }
        }


        // 設定「光棒、單數列、雙數列」的文色
        protected override void OnRowDataBound(System.Web.UI.WebControls.GridViewRowEventArgs e)
        {
            // 背景顏色暫時寫死在下面這三行，但如此一來，即無法由程序員自訂顏色；
            // 若要開放讓程序員自訂顏色，必須將下列三行註解掉，亦即改由每支程式的前端 .aspx 透過 GridView 的「屬性」視窗，去設定顏色
            Wizard_OddRowBackColor = System.Drawing.Color.FromName("#FFFFFF");        // 白色
            Wizard_EvenRowBackColor = System.Drawing.Color.FromName("#F7F6F3");       // 淺藍色            
            Wizard_MouseOverBackColor = System.Drawing.Color.FromName("#FFFF77");     // 米黃色


            if (e.Row.RowType == DataControlRowType.DataRow)
            {
                //e.Row.ForeColor = System.Drawing.Color.Orange;

                if (e.Row.RowIndex % 2 == 0)    // 單數列
                {
                    if (FOddRowBackColor != System.Drawing.Color.Empty)
                    {
                        e.Row.Style[HtmlTextWriterStyle.BackgroundColor] = ColorTranslator.ToHtml(FOddRowBackColor);
                    }

                    if (FMouseOverBackColor != System.Drawing.Color.Empty)
                    {
                        // 滑鼠移至資料列上的顏色
                        e.Row.Attributes["onmouseover"] = "this.style.backgroundColor='" + System.Drawing.ColorTranslator.ToHtml(FMouseOverBackColor) + "'";
                        //e.Row.Attributes["onmouseover"] = "this.style.color='" + System.Drawing.ColorTranslator.ToHtml(System.Drawing.Color.Orange) + "'";
                        // 滑鼠離開資料列上的顏色
                        e.Row.Attributes["onmouseout"] = "this.style.backgroundColor='" + System.Drawing.ColorTranslator.ToHtml(FOddRowBackColor) + "'";
                        //e.Row.Attributes["onmouseover"] = "this.style.color='" + System.Drawing.ColorTranslator.ToHtml(System.Drawing.Color.Black) + "'";
                    }
                }
                else    // 雙數列
                {
                    if (FEvenRowBackColor != System.Drawing.Color.Empty)
                    {
                        e.Row.Style[HtmlTextWriterStyle.BackgroundColor] = ColorTranslator.ToHtml(FEvenRowBackColor);
                    }

                    if (FMouseOverBackColor != System.Drawing.Color.Empty)
                    {
                        // 滑鼠移至資料列上的顏色
                        e.Row.Attributes["onmouseover"] = "this.style.backgroundColor='" + System.Drawing.ColorTranslator.ToHtml(FMouseOverBackColor) + "'";
                        //e.Row.Attributes["onmouseover"] = "this.style.color='" + System.Drawing.ColorTranslator.ToHtml(System.Drawing.Color.Orange) + "'";
                        // 滑鼠離開資料列上的顏色
                        e.Row.Attributes["onmouseout"] = "this.style.backgroundColor='" + System.Drawing.ColorTranslator.ToHtml(FEvenRowBackColor) + "'";
                        //e.Row.Attributes["onmouseover"] = "this.style.color='" + System.Drawing.ColorTranslator.ToHtml(System.Drawing.Color.Black) + "'";
                    }
                }
            }
            else if (e.Row.RowType == DataControlRowType.Pager)
            {
                e.Row.BackColor = System.Drawing.Color.White;
                e.Row.HorizontalAlign = HorizontalAlign.Right;

            }
            else if (e.Row.RowType == DataControlRowType.Header)
            {
                e.Row.BackColor = System.Drawing.Color.FromName("#E7E7E7");     // 標題背景色: 淺藍色
                e.Row.Font.Bold = true;
                e.Row.HorizontalAlign = HorizontalAlign.Center;
            }
            else if (e.Row.RowType == DataControlRowType.Footer)
            {
                e.Row.BackColor = System.Drawing.Color.FromName("#5D7B9D");     // 底端背景色: 暗藍色
                e.Row.Font.Bold = true;
            }

            base.OnRowDataBound(e);

        } // end of OnRowDataBound()



        //// 光棒
        //protected override void OnDataBound(System.Web.UI.WebControls.GridViewRowEventArgs e)
        //{

        //}

        //protected override void OnRowCreated(GridViewRowEventArgs e)
        //{
        //    if (e.Row.RowType == DataControlRowType.Pager)
        //    {
        //        //LinkButton First = new LinkButton();
        //        //LinkButton Prev = new LinkButton();
        //        //LinkButton Next = new LinkButton();
        //        //LinkButton Last = new LinkButton();
        //        HtmlInputText InputPage = new HtmlInputText();
        //        Button GoPage = new Button();

        //        TableCell tc = new TableCell();

        //        //e.Row.Controls.Clear();

        //        tc.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
        //        if (_recordCount.HasValue)
        //        {
        //            tc.Controls.Add(new LiteralControl("<span style='font-size:9pt'>總共筆數：</span><span style='color:red;font-size:9pt'>" + _recordCount.ToString() + "</span>"));
        //            tc.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
        //            tc.Controls.Add(new LiteralControl("<span style='font-size:9pt'>每頁筆數：</span><span style='color:red;font-size:9pt'>" + PageSize.ToString() + "</span>"));
        //            tc.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
        //        }
        //        tc.Controls.Add(new LiteralControl("<span style='font-size:9pt'>目前：</span><span style='color:red;font-size:9pt'>" + (PageIndex + 1).ToString()));
        //        tc.Controls.Add(new LiteralControl("/"));
        //        tc.Controls.Add(new LiteralControl(PageCount.ToString() + "</span>"));
        //        tc.Controls.Add(new LiteralControl("&nbsp;&nbsp;&nbsp;&nbsp;"));

        //        InputPage.ID = "txtPageIndex";
        //        InputPage.Style.Add(HtmlTextWriterStyle.Width, "20px");
        //        tc.Controls.Add(InputPage);

        //        GoPage.Text = "GO";
        //        GoPage.ID = "GO";
        //        GoPage.Click += new System.EventHandler(OnGoPage_Click);
        //        tc.Controls.Add(GoPage);

        //        tc.ColumnSpan = this.Columns.Count;
        //        e.Row.Controls.Add(tc);

        //        InputPage.Attributes.Add("onkeypress", "if(event.keyCode==13){ this.nextSibling.click();event.returnValue=false;}");
        //    }

        //      base.OnRowCreated(e);            
        //}


        private void OnGoPage_Click(object sender, System.EventArgs e)
        {
            HtmlInputText tb = null;
            //tb = (TextBox)this.Page.FindControl("txtPageIndex");

            tb = (HtmlInputText)this.TopPagerRow.FindControl("txtPageIndex");           // 頂端頁碼列中，輸入「頁次」的控件
            //tb2 = (HtmlInputText)this.BottomPagerRow.FindControl("txtPageIndex");     // 底部頁碼列中，輸入「頁次」的控件

            int pageIndex = -1;     // 欲前往的頁次

            if (int.TryParse(tb.Value.Trim(), out pageIndex))
            {
                pageIndex = pageIndex - 1;
                if (pageIndex > -1)
                {
                    //this.PageIndex = pageIndex;

                    //int jjjj1 = this.SelectArguments.StartRowIndex;
                    //int jjjj2 = this.PageSize;
                    //string jjjj3 = _recordCount.ToString();
                    //int jjjj4 = this.PageCount;

                    // 若使用者輸入的頁數，大於既有的總頁數，則將 GridView 移至最大頁數
                    if (pageIndex > this.PageCount - 1)
                    {
                        this.PageIndex = this.PageCount - 1;
                    }
                    else  // 若使用者輸入合理的頁數(小於或等於既有的總頁數)
                    {
                        this.PageIndex = pageIndex;
                    }
                }
            }

            tb.Value = string.Empty;    // 清空使用者輸入的頁數
            //tb2.Value = string.Empty;    // 清空使用者輸入的頁數
        }


        protected override void OnLoad(EventArgs e)
        {
            // 捉取 ObjectDataSource
            ObjectDataSource ods = this.Page.FindControl(this.DataSourceID) as ObjectDataSource;    //捉不到
            //ObjectDataSource ods = this.Page.FindControl("ObjectDataSource1") as ObjectDataSource;

            //DataSourceSelectArguments dd = new DataSourceSelectArguments();
            ////DataSourceSelectArguments dssa = new DataSourceSelectArguments(4, 3);

            //int ii1 = dd.StartRowIndex;
            //int ii2 = dd.TotalRowCount;            
            //int jj1 = SelectArguments.StartRowIndex;    // GridView 啟始擷取的索引
            //int jj2 = SelectArguments.MaximumRows;      // GridView 每頁顯示的筆數
            //bool jj3 = SelectArguments.RetrieveTotalRowCount;
            //string jj4 = SelectArguments.SortExpression;
            //int jj5 = SelectArguments.TotalRowCount;

            if (ods != null)
            {
                //ViewState["pp2"] = "ods is not null";
                ods.Selected += new ObjectDataSourceStatusEventHandler(ods_Selected);
            }
            //else
                //ViewState["pp2"] = "ods is null";

                base.OnLoad(e);
        }

        /// 計算總記錄數
        private void ods_Selected(object sender, ObjectDataSourceStatusEventArgs e)
        {
            if (e.ReturnValue is IListSource)
            {
                _recordCount = ((IListSource)e.ReturnValue).GetList().Count;
                //ViewState["ppppp"] = _recordCount;
            }

            // 若在 code-behind，可這樣捉到「總筆數」
            //int intTotalNum = ((DataSet)e.ReturnValue).Tables[0].Rows.Count;
            //if ((intTotalNum > 0))
            //{
            //    lblTotalNum.Text = "共 " + intTotalNum + " 筆";
            //}
            //else
            //{
            //    lblTotalNum.Text = "共 0 筆";
            //}
        }
        

        // 是否要啟用分頁(預設為啟用)
        public bool WizardEnableCustomPager
        {
            // 若前端 .aspx 未設定 WizardGridView 的 WizardEnableCustomPager 屬性 (是否要啟用分頁)，則預設為 true (啟用)
            get { return (bool?)ViewState["WizardEnableCustomPager"] ?? true; }
            set { ViewState["WizardEnableCustomPager"] = value; }
        }


        // 分頁-UI
        protected override void InitializePager(GridViewRow row, int columnSpan, PagedDataSource pagedDataSource)
        {
            if (WizardEnableCustomPager)
                CreateCustomPager(row, columnSpan, pagedDataSource);
            else
                base.InitializePager(row, columnSpan, pagedDataSource);
        }


        // 分頁-UI (每頁要顯示幾筆，就會進入這個函數幾次)
        protected virtual void CreateCustomPager(GridViewRow row, int columnSpan, PagedDataSource pagedDataSource)
        {
            int pageCount = pagedDataSource.PageCount;              // 頁面總數 (透過 SP，仍可取得正確總頁數)
            int pageIndex = pagedDataSource.CurrentPageIndex + 1;   // 頁面索引 (目前所在頁數)
            int pageButtonCount = PagerSettings.PageButtonCount;    // 巡覽區顯示的頁面按鈕數目 (1)

            _recordCount = pagedDataSource.DataSourceCount;     // 取得 ObjectDataSource 裡撈得的 總筆數，指派給共用變數 (可從SP取得正確總筆數)

            TableCell cell = new TableCell();
            row.Cells.Add(cell);
            if (columnSpan > 1) cell.ColumnSpan = columnSpan;
            //cell.ColumnSpan = 15;

            HtmlGenericControl pager = new HtmlGenericControl("div");
            pager.Attributes["class"] = "pagination";
            cell.Controls.Add(pager);

            if (_recordCount.HasValue)
            {
                int intPageLowerRecord = pageIndex * PageSize + 1 - PageSize;   // 本頁第一筆的索引
                int intPageUpperRecord = pageIndex * PageSize;                  // 本頁最後一筆的索引

                // 檢查 GridView 最後一頁的筆數，是否為零頭，抑或剛好等於 PageSize 的數目
                int intLastPageRecord = (int)_recordCount % PageSize;

                // 若最後一頁的筆數為零頭 (不是剛好為 PageSize 的數目)，且使用者已進到了最後一頁
                if ((intLastPageRecord > 0) && (pageIndex == pageCount))
                {
                    intPageUpperRecord = pageIndex * PageSize - PageSize + intLastPageRecord;
                }

                // 目前寫法，將頁碼列的資料拆成兩行顯示，以避免因頁碼列資料過多(過長)，會造成 GridView 中的內容偶爾會發生排版異常之現象
                cell.Controls.Add(new LiteralControl("<span style='font-size:9pt'>总笔数：</span><span style='color:red;font-size:9pt'>" + _recordCount.ToString() + "</span>"));
                cell.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
                cell.Controls.Add(new LiteralControl("<span style='font-size:9pt'>  本页笔数：</span><span style='color:red;font-size:9pt'>" + intPageLowerRecord + " - " + intPageUpperRecord + "</span>"));
                cell.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
                //cell.Controls.Add(new LiteralControl("<span style='font-size:9pt'>每頁筆數：</span><span style='color:red;font-size:9pt'>" + PageSize.ToString() + "</span>"));
                //cell.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
                cell.Controls.Add(new LiteralControl("<span style='font-size:9pt'>  页次：</span><span style='color:red;font-size:9pt'>" + pageIndex.ToString() + " / " + pageCount.ToString() + "</span>"));
                cell.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
                //cell.Controls.Add(new LiteralControl("<span style='font-size:9pt'>總頁數：</span><span style='color:red;font-size:9pt'>" + pageCount.ToString() + "</span>"));
                //cell.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
                cell.Controls.Add(new LiteralControl("<span style='font-size:9pt'>  前往页次</span>"));

                HtmlInputText InputPage = new HtmlInputText();
                Button GoPage = new Button();
                GoPage.Height = 20;
                InputPage.ID = "txtPageIndex";
                InputPage.Style.Add(HtmlTextWriterStyle.Width, "20px");
                InputPage.Style.Add(HtmlTextWriterStyle.Height, "12px");
                cell.Controls.Add(InputPage);

                GoPage.ID = "GO";
                GoPage.Text = "GO";
                GoPage.Font.Size = System.Web.UI.WebControls.FontUnit.Smaller;
                GoPage.ToolTip = "单击此按钮，移至欲前往页次";
                //GoPage.Visible = false;
                GoPage.Click += new System.EventHandler(OnGoPage_Click);
                cell.Controls.Add(GoPage);

                cell.ColumnSpan = this.Columns.Count;

                // 若使用者按的是 Enter 鍵
                InputPage.Attributes.Add("onkeypress", "if(event.keyCode==13){ this.nextSibling.click();event.returnValue=false;}");

                // 此種寫法，可將頁碼列中的訊息，從兩行合併成一行。但若頁碼列資料過多(過長)，會造成 GridView 中的內容偶爾會發生排版異常之現象
                //pager.Controls.Add(new LiteralControl("<span style='font-size:9pt'>總共筆數：</span><span style='color:red;font-size:9pt'>" + _recordCount.ToString() + "</span>"));
                //pager.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
                //pager.Controls.Add(new LiteralControl("<span style='font-size:9pt'>目前筆數：</span><span style='color:red;font-size:9pt'>" + intPageLowerRecord + " - " + intPageUpperRecord + "</span>"));
                //pager.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
                //pager.Controls.Add(new LiteralControl("<span style='font-size:9pt'>每頁筆數：</span><span style='color:red;font-size:9pt'>" + PageSize.ToString() + "</span>"));
                //pager.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
                //pager.Controls.Add(new LiteralControl("<span style='font-size:9pt'>目前頁數：</span><span style='color:red;font-size:9pt'>" + pageIndex.ToString() + "</span>"));
                //pager.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
                //pager.Controls.Add(new LiteralControl("<span style='font-size:9pt'>總頁數：</span><span style='color:red;font-size:9pt'>" + pageCount.ToString() + "</span>"));
                pager.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
            }

            if (pageCount >= 0)
            {
                //TableCell cell2 = new TableCell();
                //row.Cells.Add(cell2);

                //HtmlGenericControl pager = new HtmlGenericControl("div");
                //pager.Attributes["class"] = "pagination";
                //cell.Controls.Add(pager);

                int min = pageIndex - pageButtonCount;
                int max = pageIndex + pageButtonCount;

                if (max > pageCount)
                    min -= max - pageCount;
                else if (min < 1)
                    max += 1 - min;

                Control page = null;

                // 建立「第一頁」LinkButton
                page = pageIndex > 1
                                ? BuildLinkButton(pageIndex - 2, PagerSettings.FirstPageText, "Page", "First")
                                : BuildSpan(PagerSettings.FirstPageText, "disabled");
                //pager.Controls.Add(page); // 這個 LinkButton 若也加入，在 GridView 中有時換頁會導致排版不正常、欄位寬度亂變、無法控制寬度

                // 建立「上一頁」LinkButton
                page = pageIndex > 1
                                ? BuildLinkButton(pageIndex - 2, PagerSettings.PreviousPageText, "Page", "Prev")
                                : BuildSpan(PagerSettings.PreviousPageText, "disabled");
                pager.Controls.Add(page);


                // 建立「頁碼」LinkButton
                bool needDiv = false;
                for (int i = 1; i <= pageCount; i++)
                {
                    if (i <= 2 || i > pageCount - 2 || (min <= i && i <= max))
                    {
                        string text = i.ToString(NumberFormatInfo.InvariantInfo);
                        page = i == pageIndex
                                ? BuildSpan(text, "current")
                                : BuildLinkButton(i - 1, text, "Page", text);
                        pager.Controls.Add(page);
                        needDiv = true;
                    }
                    else if (needDiv)
                    {
                        page = BuildSpan("&hellip;", null);
                        pager.Controls.Add(page);
                        needDiv = false;
                    }
                }


                // 建立「下一頁」LinkButton
                page = pageIndex < pageCount
                        ? BuildLinkButton(pageIndex, PagerSettings.NextPageText, "Page", "Next")
                        : BuildSpan(PagerSettings.NextPageText, "disabled");
                pager.Controls.Add(page);


                // 建立「最後一頁」LinkButton
                page = pageIndex < pageCount
                                ? BuildLinkButton(pageIndex, PagerSettings.LastPageText, "Page", "Last")
                                : BuildSpan(PagerSettings.LastPageText, "disabled");
                //pager.Controls.Add(page); // 這個 LinkButton 若也加入，在 GridView 中有時換頁會導致排版不正常、欄位寬度亂變、無法控制寬度


                // 寫死分頁按鈕的外觀、特性，不讓開發人員由前端 .aspx 自訂
                PagerSettings.FirstPageText = "首页";
                PagerSettings.PreviousPageText = "上一页";
                PagerSettings.NextPageText = "下一页";
                PagerSettings.LastPageText = "尾页";

                // 設定「頁碼列」，是要顯示在 GridView 上方還下方，還是上、下方都顯示；另再設定「頁碼列」要顯示在左、中、右側。
                // 若在此寫死，則程序員即無法由前端自訂；若要開放讓程序員自訂位置，必須將下面這兩行註解掉。
                //PagerSettings.Position = PagerPosition.Top;   //.TopAndBottom;
                //PagerStyle.HorizontalAlign = HorizontalAlign.Right;
                
                PagerSettings.PageButtonCount = 2;
            }

        } // end of CreateCustomPager()


        // 分頁-UI
        private Control BuildLinkButton(int pageIndex, string text, string commandName, string commandArgument)
        {
            //LinkButton link = new LinkButton();
            WizardPagerLinkButton link = new WizardPagerLinkButton(this);
            link.Text = text;
            link.EnableCallback(ParentBuildCallbackArgument(pageIndex));
            link.CommandName = commandName;
            link.CommandArgument = commandArgument;
            return link;
        }


        // 分頁-UI
        private Control BuildSpan(string text, string cssClass)
        {
            HtmlGenericControl span = new HtmlGenericControl("span");
            if (!String.IsNullOrEmpty(cssClass)) span.Attributes["class"] = cssClass;
            span.InnerHtml = text;
            return span;
        }


        // 分頁-UI
        private string ParentBuildCallbackArgument(int pageIndex)
        {
            MethodInfo m =
                typeof(GridView).GetMethod("BuildCallbackArgument", BindingFlags.NonPublic | BindingFlags.Instance, null,
                                            new Type[] { typeof(int) }, null);
            return (string)m.Invoke(this, new object[] { pageIndex });
        }


    } // end of class
} // end of namespace
